@@ -6,6 +6,7 @@ to interact with the parse tree.
 
 """
 
+import copy
 import logging
 from typing import Any, Dict, List, Optional, Sequence, Type, Union, cast
 
@@ -306,6 +307,7 @@ class DictParser:
             A generator of serialized string chunks
 
         """
+
         if isinstance(value, str):
             return self.parse_pair(key, value)
         elif isinstance(value, (list, tuple)):
@@ -314,11 +316,12 @@ class DictParser:
             else:
                 return self.parse_list(key, value)
         elif isinstance(value, dict):
+            to_parse = copy.copy(value)
             if key in KEYS_WITH_NAME_FIELDS or "name" not in value.keys():
                 name = None
             else:
-                name = value.pop("name")
-            return self.parse_block(key, value, name)
+                name = to_parse.pop("name")
+            return self.parse_block(key, to_parse, name)
         else:
             raise TypeError("Value must be a string, list, tuple, or dict.")
 
@@ -475,8 +478,8 @@ class DictParser:
 
         """
         if force_quote or key in QUOTED_LITERAL_KEYS:
-            return QuotedSyntaxToken(value, prefix, suffix)
+            return QuotedSyntaxToken(value, prefix=prefix, suffix=suffix)
         elif key in EXPR_BLOCK_KEYS:
-            return ExpressionSyntaxToken(value.strip(), prefix, suffix)
+            return ExpressionSyntaxToken(value.strip(), prefix=prefix, suffix=suffix)
         else:
-            return SyntaxToken(value, prefix, suffix)
+            return SyntaxToken(value, prefix=prefix, suffix=suffix)
