@@ -31,6 +31,7 @@ class SyntaxToken:
     """
 
     value: str
+    line_number: Optional[int] = None
     prefix: str = ""
     suffix: str = ""
 
@@ -42,6 +43,14 @@ class SyntaxToken:
         """Accepts a visitor and calls the visitor's token method on itself."""
         return visitor.visit_token(self)
 
+    def __eq__(self, other: Any) -> bool:
+        if isinstance(other, self.__class__):
+            return self.__dict__ == other.__dict__
+        elif isinstance(other, str):
+            return self.value == other
+        else:
+            return NotImplemented
+
     def __str__(self) -> str:
         return items_to_str(self.prefix, self.format_value(), self.suffix)
 
@@ -107,6 +116,12 @@ class SyntaxNode(ABC):
         """Returns all child SyntaxNodes, but not SyntaxTokens."""
         ...
 
+    @property
+    @abstractmethod
+    def line_number(self) -> Optional[int]:
+        """Returns the line number of the first SyntaxToken in the node"""
+        ...
+
     @abstractmethod
     def accept(self, visitor: Visitor) -> Any:
         """Accepts a Visitor that can interact with the node.
@@ -144,6 +159,10 @@ class PairNode(SyntaxNode):
     def children(self) -> None:
         return None
 
+    @property
+    def line_number(self) -> Optional[int]:
+        return self.type.line_number
+
     def accept(self, visitor: Visitor) -> Any:
         """Accepts a visitor and calls the visitor's pair method on itself."""
         return visitor.visit_pair(self)
@@ -179,13 +198,16 @@ class ListNode(SyntaxNode):
         return f"{self.__class__.__name__}(type='{self.type.value}')"
 
     @property
-    def children(self,) -> Optional[Tuple[PairNode, ...]]:
-        if isinstance(self.items[0], PairNode):
+    def children(self) -> Tuple[PairNode, ...]:
+        if self.items and isinstance(self.items[0], PairNode):
             # Assume that all elements are pairs
-            self.items = cast(Tuple[PairNode, ...], self.items)  # type: ignore
-            return self.items
+            return cast(Tuple[PairNode, ...], self.items)
         else:
-            return None
+            return tuple()
+
+    @property
+    def line_number(self) -> Optional[int]:
+        return self.type.line_number
 
     def accept(self, visitor: Visitor) -> Any:
         """Accepts a visitor and calls the visitor's list method on itself."""
@@ -203,75 +225,6 @@ class ListNode(SyntaxNode):
         )
 
 
-@dataclass(frozen=True)
-class BlockNode(SyntaxNode):
-    """A LookML block, enclosed in curly braces. Like ``view`` or ``dimension``.
-
-    Attributes:
-        type: The field type, the value that precedes the colon.
-        left_brace: A syntax token for the opening brace "{".
-        right_brace: A syntax token for the closing brace "}".
-        colon: An optional Colon SyntaxToken. If not supplied, a default colon is
-            created with a single space suffix after the colon.
-        name: An optional name token, the value that follows the colon.
-        container: A container node that holds the block's child nodes.
-
-    """
-
-    type: SyntaxToken
-    left_brace: LeftCurlyBrace
-    right_brace: RightCurlyBrace
-    colon: Optional[Colon] = Colon(suffix=" ")
-    name: Optional[SyntaxToken] = None
-    container: Optional[ContainerNode] = None
-
-    def __repr__(self) -> str:
-        name = f"name='{self.name.value}'" if self.name else None
-        return f"{self.__class__.__name__}(type='{self.type.value}', {name})"
-
-    @property
-    def children(self) -> Optional[Tuple[ContainerNode, ...]]:
-        return (self.container,) if self.container else None
-
-    def accept(self, visitor: Visitor) -> Any:
-        """Accepts a visitor and calls the visitor's block method on itself."""
-        return visitor.visit_block(self)
-
-    def __str__(self) -> str:
-        name = self.name or ""
-        container = self.container or ""
-        return items_to_str(
-            self.type, self.colon, name, self.left_brace, container, self.right_brace
-        )
-
-
-@dataclass(frozen=True)
-class DocumentNode(SyntaxNode):
-    """The root node of the parse tree.
-
-    Attributes:
-        container: The top-level container node.
-        prefix: Leading whitespace or comments before the document.
-        suffix: Trailing whitespace or comments after the document.
-
-    """
-
-    container: ContainerNode
-    prefix: str = ""
-    suffix: str = ""
-
-    @property
-    def children(self) -> Tuple[ContainerNode]:
-        return (self.container,)  # type: ignore
-
-    def accept(self, visitor: Visitor) -> Any:
-        """Accepts a visitor and calls the visitor's visit method on itself."""
-        return visitor.visit(self)
-
-    def __str__(self) -> str:
-        return items_to_str(self.prefix, self.container, self.suffix)
-
-
 @dataclass(frozen=True)
 class ContainerNode(SyntaxNode):
     """A sequence of nodes, either at the top level of a document, or within a block.
@@ -304,6 +257,14 @@ class ContainerNode(SyntaxNode):
     def children(self) -> Tuple[Union[BlockNode, PairNode, ListNode], ...]:
         return self.items
 
+    def line_number(self) -> Optional[int]:
+        try:
+            first_item = self.items[0]
+        except IndexError:
+            return None
+        else:
+            return first_item.line_number
+
     def accept(self, visitor: Visitor) -> Any:
         """Accepts a visitor and calls the visitor's container method on itself."""
         return visitor.visit_container(self)
@@ -314,6 +275,83 @@ class ContainerNode(SyntaxNode):
         return items_to_str(*self.items)
 
 
+@dataclass(frozen=True)
+class BlockNode(SyntaxNode):
+    """A LookML block, enclosed in curly braces. Like ``view`` or ``dimension``.
+
+    Attributes:
+        type: The field type, the value that precedes the colon.
+        left_brace: A syntax token for the opening brace "{".
+        right_brace: A syntax token for the closing brace "}".
+        colon: An optional Colon SyntaxToken. If not supplied, a default colon is
+            created with a single space suffix after the colon.
+        name: An optional name token, the value that follows the colon.
+        container: A container node that holds the block's child nodes.
+
+    """
+
+    type: SyntaxToken
+    left_brace: LeftCurlyBrace = LeftCurlyBrace(suffix="\n")
+    right_brace: RightCurlyBrace = RightCurlyBrace(prefix="\n")
+    colon: Optional[Colon] = Colon(suffix=" ")
+    name: Optional[SyntaxToken] = None
+    container: ContainerNode = ContainerNode(items=tuple())
+
+    def __repr__(self) -> str:
+        name = f"name='{self.name.value}'" if self.name else None
+        return f"{self.__class__.__name__}(type='{self.type.value}', {name})"
+
+    @property
+    def children(self) -> Tuple[Union[BlockNode, PairNode, ListNode], ...]:
+        return self.container.children
+
+    @property
+    def line_number(self) -> Optional[int]:
+        return self.type.line_number
+
+    def accept(self, visitor: Visitor) -> Any:
+        """Accepts a visitor and calls the visitor's block method on itself."""
+        return visitor.visit_block(self)
+
+    def __str__(self) -> str:
+        name = self.name or ""
+        container = self.container or ""
+        return items_to_str(
+            self.type, self.colon, name, self.left_brace, container, self.right_brace
+        )
+
+
+@dataclass(frozen=True)
+class DocumentNode(SyntaxNode):
+    """The root node of the parse tree.
+
+    Attributes:
+        container: The top-level container node.
+        prefix: Leading whitespace or comments before the document.
+        suffix: Trailing whitespace or comments after the document.
+
+    """
+
+    container: ContainerNode
+    prefix: str = ""
+    suffix: str = ""
+
+    @property
+    def children(self) -> Tuple[ContainerNode]:
+        return (self.container,)
+
+    @property
+    def line_number(self) -> Optional[int]:
+        return 1  # Document always starts on the first line
+
+    def accept(self, visitor: Visitor) -> Any:
+        """Accepts a visitor and calls the visitor's visit method on itself."""
+        return visitor.visit(self)
+
+    def __str__(self) -> str:
+        return items_to_str(self.prefix, self.container, self.suffix)
+
+
 class Visitor(ABC):
     """Abstract base class for visitors that interact with the parse tree."""
 
